#include "common.h"

static const char title_string[] = STR_TITLE;

const char umd_real_name[] = STR_UMD_DISC;
const char umd_search_dir[256] = "ms0:/ISO";
const char umd_search_dir2[256] = "ms0:/UMDEMULATOR/ISO";
const char str_tab[] = "";

/////////////////////////////////////////////////////////////////////////////
// setup 
/////////////////////////////////////////////////////////////////////////////
int boot_mode = 0;

char target_path[256];

/////////////////////////////////////////////////////////////////////////////
// setting parameters
/////////////////////////////////////////////////////////////////////////////
int top_menu_sel = 0;
int sel = 1;
int devkit;
int mjr,mnr,rev;
int speed_sel = 2;
int volt;
int umd_ver_sel;
int remote_sel = 0;

int om_sel = 0;

char buf[256*32];
char *file_list[32];
char *full_list[32];

static const u32 preload_addr_sel[] = {
0x88e00000,
0x89000000,
0x89200000,
0x89400000,
0x89600000,
0x89800000};

#define NUM_PRELOAD_ADDR (sizeof(preload_addr_sel)/sizeof(u32))
static char preload_menu_buf[NUM_PRELOAD_ADDR][16];
static const char *preload_menu[NUM_PRELOAD_ADDR+1];

/*******************************************************************************
	top menu
*********************************************************************************/

const char *top_menu[] =
{
STR_TM_START,
STR_TM_UMDS,
STR_TM_FIRM,
STR_TM_CLK,
STR_TM_BOOT,
STR_TM_FLASH,
STR_TM_OTHER,
STR_TM_EXIT,
NULL};

enum {
TM_START,
TM_UMD,
TM_FIRM,
TM_CPUCLOCK,
TM_TARGET,
TM_FLASH,
TM_OTHER,
TM_EXIT
};

const char *target_menu[] =
{
STR_TG_XMB,
STR_TG_UMD,
STR_TG_LOAD,
STR_TG_MPHGL,
STR_TG_RUMD,
NULL};

const char *umd_mount_menu[] =
{
STR_UM_ANYUMD,
STR_UM_NOUMD,
NULL};

const char *umd_ver_menu[] =
{
STR_VM_NOCHANGE,
STR_VM_2XX200,
NULL};

const char *speed_menu[] =
{
" 66MHz / 37MHz",
"111MHz / 54MHz",
"222MHz /111MHz",
"333MHz /166MHz",
STR_CK_NORMAL,
NULL};

const char *flash_menu[] =
{
"",
"1.50       ",
"2.00       ",
"2.50       ",
"2.60       ",
"2.71       ",
//"1.00 (not supported)",
NULL};

static const char *flash_root[] =
{
	"",
	"ms0:/dh/150",
	"ms0:/dh/200",
	"ms0:/dh/250",
	"ms0:/dh/260",
	"ms0:/dh/271",
//	"ms0:/dh/100",
	NULL
};

static const char *boot_mode_str[] =
{
STR_BM_XMB,
STR_BM_UMD,
STR_BM_MS,
STR_BM_RUN,
NULL};

//
/*
bus : 37Mhz limit
CPU : variable
*/

static const char *NANDflash_menu[] =
{
STR_F1_BACKUP,
str_tab,
STR_F1_INST,
STR_F1_CLEAN,
str_tab,
STR_F0_FONT,
STR_F0_KDRES,
STR_F0_CLEAN,
NULL};

enum{
	F1_BACKUP,
	FL_TAB1,
	F1_INST,
	F1_CLEAN,
	FL_TAB2,
	F0_FONT,
	F0_KDRES,
	F0_CLEAN,
	FL_RETURN
};

/////////////////////////////////////////////////////////////////////////////
// UMD select
/////////////////////////////////////////////////////////////////////////////
static int sel_umd(int ypos)
{
	char *ptr;
	int sel;

	// real UMD added
	file_list[0] = (char *)umd_real_name;
	file_list[1] = 0;

	ptr = buf;

	ptr = add_file_list(umd_search_dir ,file_list,full_list,ptr,NULL);
	ptr = add_file_list(umd_search_dir2,file_list,full_list,ptr,NULL);

	// no file ?
	if(file_list[0] == 0)
		return 0;

	// choise one
	sel = selitem(0,ypos,(const char **)file_list,1);

	if(sel==0)
	{
		// UMD DISC MODE
		dh_config.umd_path[0] = 0; // real UMD
		dh_config.flags &= ~(DH_FLAG_ANYUMD|DH_FLAG_NOUMD);
		//umd_lock = 2;
	}
	else  if(sel>0)
	{
		// UMD image
		strcpy(dh_config.umd_path,full_list[sel]);

		// FW1.50 choice MOUNT mode
		if(memcmp("ms0:/dh/150",dh_config.flash,11)==0)
			sel = selitem(0,7,umd_mount_menu,0); // FW 1.50
		else
			sel = 0; // ANYUMD only

		dh_config.flags &= ~(DH_FLAG_ANYUMD|DH_FLAG_NOUMD);
		switch(sel)
		{
		case 1: // NOUMD
			dh_config.flags |= DH_FLAG_NOUMD;
			break;
		case 0: // ANYUMD
		default:
			dh_config.flags |= DH_FLAG_ANYUMD;
		}
	}
	return 1;
}

/////////////////////////////////////////////////////////////////////////////
// FLASH maintenance
/////////////////////////////////////////////////////////////////////////////
static int sel_flash(int ypos)
{
	int sel = selitem(0,ypos,NANDflash_menu,0);
	switch(sel)
	{
	case F1_BACKUP:	flash1_backup();	break;
	case F1_INST:	flash1_install();	break;
	case F1_CLEAN:	flash1_cleanup();	break;
	case F0_FONT:	flash0_font();		break;
	case F0_KDRES:	flash0_kd_resource();break;
	case F0_CLEAN:	flash0_cleanup();	break;
	}
	return 1;
}

/*******************************************************************************
	other menu
*********************************************************************************/

const char *other_menu[] =
{
STR_OM_UMDVER,
STR_OM_REMOTE,
STR_OM_PRELOAD,
str_tab,
STR_OM_RETURN,
NULL};

enum{
OM_UMDVER,
OM_REMOTE,
OM_PRELOAD,
OM_TAB1,
OM_RETURN
};

const char *uartsel_menu[] =
{
STR_RC_REMOTE,
STR_RC_KPRINT,
NULL};

/*******************************************************************************
	check UMD image file
*********************************************************************************/
static void check_umd_image(void)
{
	if(dh_config.flags & (DH_FLAG_ANYUMD|DH_FLAG_NOUMD) )
	{
		int fd = sceIoOpen(dh_config.umd_path, PSP_O_RDONLY,0777);
		if(fd<0)
		{
			// UMD image not found , clear image file
			dh_config.umd_path[0] = 0x00;
		}
		else
		{
			// UMD image found
			sceIoClose(fd);
		}
	}
}

/*******************************************************************************
	AUTO RUN waiting
*********************************************************************************/
static int autorun_waiting(void)
{
	int fd;
	char read_buf[16];
	int autorun_time = 0;
	int timeout;

	// get autorun time from file
	fd = sceIoOpen("ms0:/dh/autorun.ini",PSP_O_RDONLY,0777);

	// no autorun
	if(fd < 0) return 0;

	if(fd > 0)
	{
		memset(read_buf,0,sizeof(read_buf));
		sceIoRead(fd,read_buf,sizeof(read_buf)-1);
		sceIoClose(fd);
		autorun_time = atoi(read_buf);
	}

	// no autorun
	if(autorun_time<0) return 0;

	// autorun waiting
	for(timeout = autorun_time*50 ; timeout >=0 ; timeout--)
	{
		locate(CENTER_X(STR_AUTORUN_WAIT),6);
	 	printf(STR_AUTORUN_WAIT,(timeout+49)/50 );

		if( get_key() & PSP_CTRL_CIRCLE) return 0;
		sceKernelDelayThread(20000);
	}

	locate(CENTER_X(STR_AUTORUN),6);
 	printf(STR_AUTORUN);

	return 1;
}

/*******************************************************************************
	draw display
*********************************************************************************/
void refresh_display(void)
{
	char path[CONSOLE_WIDTH+1];
	int x,y,y2;
	const char *umd_mode_str;

	pspDebugScreenClear();

 	locate(CENTER_X(STR_TITLE),0);
  	printf(STR_TITLE);

	// battery voltage
	char volt_str[0x20];
	volt = scePowerGetBatteryVolt();
	switch(volt)
	{
	case 0x802b0100:
		strcpy(volt_str,STR_BAT_NOCONNECT);
		break;
	default:
		if(volt>0) sprintf(volt_str,"%d.%03dV",volt/1000,volt%1000);
		else       sprintf(volt_str,STR_BAT_UNKLNOWN,volt);
	}

	// UMD mode string
	if(dh_config.flags & DH_FLAG_ANYUMD)
		umd_mode_str = STR_UM_ANYUMD;
	else if(dh_config.flags & DH_FLAG_NOUMD)
		umd_mode_str = STR_UM_NOUMD;
	else
		umd_mode_str = umd_real_name;

	// dh version
	u32 dhver = dhGetVersion();

	// show settings
	y = (CONSOLE_HEIGHT>27) ? 20 : 15;

	locate(0,y++);
	printf(STR_STS_TARGET,target_path,boot_mode_str[boot_mode]);

 	strncpy(path, dh_config.umd_path, CONSOLE_WIDTH);
 	path[CONSOLE_WIDTH] = '\0';

	locate(0,y++);
	printf(STR_STS_UMDISO,dh_config.umd_path);
	if(CONSOLE_WIDTH<96) y+=2;
	locate(0,y++);
	printf(STR_STS_UMDMODE,umd_mode_str,umd_ver_menu[umd_ver_sel]);
	locate(0,y++);
	printf(STR_STS_FLASH,dh_config.flash , (dh_config.flags & DH_FLAG_FLASH_EMU) ? "MS HOOK" : "NANDFlash");
	y2 = y;
	locate(0,y++);
	printf(STR_STS_CLOCK,dh_config.cpu_Mhz,dh_config.bus_Mhz);
	locate(0,y++);
	printf(STR_STS_REMOTE,uartsel_menu[(dh_config.flags & DH_FLAG_KPRINT_UART) ? 1 : 0]);

	locate(0,y++);
	printf(STR_STS_PREADDR,dh_config.preload_addr);

	// show status
	x = CONSOLE_WIDTH-24;
	y = y2;

	locate(x,y++);
	printf(STR_STS_DHVER,dhver>>24,(dhver>>16)&0xff,dhver&0xffff);
	locate(x,y++);
	printf(STR_STS_FWVER,mjr,mnr,rev);
	locate(x,y++);
	printf(STR_STS_BATTERY,volt_str);

	// useage
 	locate(CENTER_X(STR_STS_KEYS),CONSOLE_HEIGHT-1);
 	printf(STR_STS_KEYS);
 
	//sceDisplayWaitVblankStart();
}

/*******************************************************************************
	other menu
*********************************************************************************/

int other_menu_sel(int ypos)
{
	int refresh = 1;
	int i;
	int sel;

	// build preload address menu string
	for(i=0;i<NUM_PRELOAD_ADDR;i++)
	{
		sprintf(preload_menu_buf[i],"0x%08X",preload_addr_sel[i]);
		preload_menu[i] = preload_menu_buf[i];
	}
	preload_menu[i] = NULL;

	while(1)
	{
		if(refresh)
		{
			refresh_display();
			refresh = 0;
		}

 		locate(CENTER_X(STR_TM_OTHER),3);
		printf(STR_TM_OTHER);

		sel = selitem(0,5,other_menu,om_sel);
		if(sel<0) continue;

		om_sel = sel;
		locate(CENTER_X(other_menu[sel]),5);
		printf(other_menu[sel]);

		switch(om_sel)
		{
		case OM_UMDVER:
			sel = selitem(0,7,umd_ver_menu,umd_ver_sel);
			if(sel>=0) umd_ver_sel = sel;

			if(umd_ver_sel==0)
				dh_config.flags &= ~DH_FLAG_SFO_VER;
			else
				dh_config.flags |=  DH_FLAG_SFO_VER;
			refresh=1;
			break;

		case OM_REMOTE:
			sel = selitem(0,7,uartsel_menu,remote_sel);
			if(sel>=0)
			{
				remote_sel = sel;
				switch(remote_sel)
				{
				case 0: dh_config.flags &= ~ DH_FLAG_KPRINT_UART; break;
				case 1: dh_config.flags |=   DH_FLAG_KPRINT_UART; break;
				}
				refresh=1;
			}
			break;
		case OM_PRELOAD:
			sel = selitem(0,7,preload_menu,2); // default = 0x89200000
			if(sel>=0)
			{
				dh_config.preload_addr = preload_addr_sel[sel];
				refresh=1;
			}
			break;
		default:
			return 1;
		}
	}
}

/*******************************************************************************
	top menu
*********************************************************************************/
int fep_menu(int is_boot_with_devhook)
{
	int refresh;
	int ypos;

	devkit = sceKernelDevkitVersion();
	mjr = devkit>>24;
	mnr = (devkit>>16)&0xff;
	rev = devkit &0xffff;

	 // disable some menu when a boot with devhook
	if(is_boot_with_devhook || 1) /* disabe flash menu on 2.71 */
	{
		top_menu[TM_FLASH] = str_tab;
	}

	// celar target path
	strcpy(target_path,"");

	// SFO ver
	umd_ver_sel = (dh_config.flags & DH_FLAG_SFO_VER) ? 1 : 0;

//	strcpy(dh_config.flash,"ms0:/150");
//	dh_config.flags &= ~DH_FLAG_FLASH_EMU;

//	dh_config.cpu_Mhz = 222;
//	dh_config.bus_Mhz = 111;

	// adjust preload address
	if(
		(dh_config.preload_addr < 0x88000000) ||
		(dh_config.preload_addr > 0x9e000000) ||
		(dh_config.preload_addr & 0x0000ffff) != 0)
	{
		// default value;
		dh_config.preload_addr = 0x89200000;
	}
	
	// DISPLAY IMAGE/CLOCK INFORMATION 3sec
	dh_config.flags |= DH_FLAG_XMB_INFO;

	// umd image check
	check_umd_image();

	// show current setting
	refresh_display();

	// autorun check
	if( autorun_waiting() ) return 0;

	refresh = 1;

	while(1)
	{
		if(refresh)
		{
			refresh_display();
			refresh = 0;
		}

 		locate(CENTER_X(STR_TOPMENU),3);
		printf(STR_TOPMENU);

		if(refresh<2)
		{
			sel = selitem(0,5,top_menu,top_menu_sel);
			if(sel<0) continue;
			top_menu_sel = sel;
		}
		locate(CENTER_X(top_menu[sel]),5);
		printf(top_menu[sel]);
		locate(0,7);

		ypos = 7;

		switch(sel)
		{
		case TM_START:
			return 0;
		case TM_UMD:
			refresh = sel_umd(ypos);
			break;
		case TM_FIRM:
			sel = selitem(0,7,flash_menu,0);
			if(sel>=0)
			{
				switch(sel)
				{
				case 0: 

					//dh_config.flags &= ~DH_FLAG_FLASH_EMU;
					break;
				case 1:
					// FW1.50
					dh_config.flags |= DH_FLAG_FLASH_EMU;
					break;
				default:
					// 2.00 - 2.71
					dh_config.flags |= DH_FLAG_FLASH_EMU;

					// NOUMD -> ANYUMD
					if(dh_config.flags & DH_FLAG_NOUMD)
					{
						dh_config.flags = (dh_config.flags & ~DH_FLAG_NOUMD) | DH_FLAG_ANYUMD;
					}
					break;
				}
				// flash root dir
				strcpy(dh_config.flash,flash_root[sel]);
				refresh=1;
			}
			break;
		case TM_CPUCLOCK:
			sel = selitem(0,7,speed_menu,speed_sel);
			if(sel>=0)
			{
				int clk_val = dh_config.cpu_Mhz * 0x10000 + dh_config.bus_Mhz;
				speed_sel = sel;
				switch(sel)
				{
				case 0: clk_val =  67*0x10000 +  33; break;
				case 1: clk_val = 111*0x10000 +  55; break;
				case 2: clk_val = 222*0x10000 + 111; break;
				case 3: clk_val = 333*0x10000 + 166; break;
				case 4: clk_val =                 0; break;
				}
				dh_config.cpu_Mhz = clk_val / 0x10000;
				dh_config.bus_Mhz = clk_val & 0xffff;
				refresh=1;
				}
				break;
		case TM_TARGET:
			sel = selitem(0,7,target_menu,0);
			if(sel>=0)
			{
				boot_mode = sel;
				switch(sel)
				{
				case 0: boot_mode = 0; break;
				case 1: boot_mode = 1; strcpy(target_path,"disc0:/PSP_GAME/SYSDIR/EBOOT.BIN"); break;
				case 2: boot_mode = 3; strcpy(target_path,"disc0:/PSP_GAME/SYSDIR/EBOOT.BIN"); break;
				case 3: boot_mode = 2; strcpy(target_path,"ms0:/PSP/GAME/MPHGAMELOADER/EBOOT.PBP"); break;
				case 4: boot_mode = 2; strcpy(target_path,"ms0:/PSP/GAME/RunUMD/EBOOT.PBP"); break;
				}
				refresh=1;
			}
			break;
		case TM_FLASH:
			refresh = sel_flash(ypos);
			break;
    	case TM_OTHER:
			refresh = other_menu_sel(ypos);
			break;
		case TM_EXIT:
			boot_mode = -1;
			return 0;
		}
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// disable when devhook reboot mode
/////////////////////////////////////////////////////////////////////////////
void menu_disable_on_devhook(void)
{
}
