#include <pspsdk.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

PSP_MODULE_INFO("DevhookReboot", 0x1000, 1, 0);

/* Define the thread attribute as 0 so that the main thread does not get converted to user mode */
PSP_MAIN_THREAD_ATTR(0);

#define JAL_OPCODE	0x0C000000
#define J_OPCODE	0x08000000

#define MAKE_CALL(a, f) _sw(JAL_OPCODE | (((u32)(f) & 0x3FFFFFFF) >> 2), a); 
#define MAKE_JUMP(a, f) _sw(J_OPCODE | (((u32)(f) & 0x3FFFFFFF) >> 2), a); 


int sceKernelExitVSHVSH(void *param);

void ClearCaches()
{
	sceKernelDcacheWritebackAll();
	sceKernelIcacheClearAll();
}

// Patch ExitVSHVSH to allow it to be called in kernel mode without
// returning ILLEGAL_PERM_CALL
#define NOP 0x00000000
#define BEQ_ZERO_ZERO_P9 0x10000008


void PatchExitVSHVSH()
{
	u32 *mod = (u32 *)sceKernelFindModuleByName("sceLoadExec");
	
	if (mod)
	{
		u32 text_addr = *(mod+27);	
		_sw(BEQ_ZERO_ZERO_P9, text_addr+0x15E4);
		_sw(NOP, text_addr+0x1618);
	}
}

#define BEQ_ZERO_ZERO_PC 0x1000000b

void PatchExecVSH()
{
	u32 *mod = (u32 *)sceKernelFindModuleByName("sceLoadExec");
	
	if (mod)
	{
		u32 text_addr = *(mod+27);
		_sw(BEQ_ZERO_ZERO_PC, text_addr+0x1DBC);
		_sw(0, text_addr+0x1DFC);
	}
}

int Reboot()
{
	int k1 = pspSdkSetK1(0);

	PatchExitVSHVSH();
	ClearCaches();
	sceKernelExitVSHVSH(NULL);

	pspSdkSetK1(k1);
	return 0;
}

static unsigned int reboot_param2[0x400/4];

int sceKernelLoadExecVSHMs1(char *, void *);

/* Dummy reboot for 1.50 atm (it will show a can't start game error) */
int DummyReboot()
{
	unsigned int params[9];
	char path[32];

	int k1 = pspSdkSetK1(0);

	strcpy(path, "ms0:/dummy.prx");

	params[0] = sizeof(params);
	params[1] = path ? strlen(path)+1 : 0;
	params[2] = (unsigned int)path;
	params[3] = (unsigned int)"game";
	params[4] = 0x400;
	params[5] = (unsigned int)reboot_param2;
	params[6] = 0;
	params[7] = 0;
	params[8] = 0;

	memset(reboot_param2,0,sizeof(reboot_param2));
	reboot_param2[0] = sizeof(reboot_param2);
	reboot_param2[1] = 0x00000020;
	reboot_param2[0x90] = 1;
	reboot_param2[0x91] = 4;
	reboot_param2[0x92] = 0x13;
	reboot_param2[0xa0] = 1;	

	PatchExecVSH();
	ClearCaches();
	
	sceKernelLoadExecVSHMs1(path, &params);

	pspSdkSetK1(k1);

	return 0;
}

/* DEstroy HEN reboot system. Now devhook will take the control of reboot */

#define LUI_AT_88c0	0x3C0188c0

int DestroyHen()
{
	int k1 = pspSdkSetK1(0);

	u32 *mod = (u32 *)sceKernelFindModuleByName("sceLoadExec");
	
	if (mod)
	{	
		/* Restore call to reboot.bin */
		u32 text_addr = *(mod+27);
		_sw(LUI_AT_88c0, text_addr+0x2578);
		/* Restore call to 2RLZ decompress */
		MAKE_CALL(text_addr+0x253C, text_addr+0x2978);
		ClearCaches();
	}

	pspSdkSetK1(k1);

	return 0;
}

int module_start(SceSize args, void *argp)
{
	return 0;
}

int module_stop(void)
{
	return 0;
}
